//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "CUtils.h"

//////////////////////////////////////////////////////////////////////////////
double medianbiasfactor(const int nn){
//////////////////////////////////////////////////////////////////////////////

  const int nmax = 1000;
  double ans = 1.0;
  int n = (nn - 1)/2;
  
  if(nn >= nmax) return LN2;// asymptotic

  for (int i = 1; i <= n; ++i){
    ans -= 1.0/(double)(2*i);
    ans += 1.0/(double)(2*i + 1);
  }

  return ans;
}
//////////////////////////////////////////////////////////////////////////////
vector<string> SplitString(const string stringtodivide, const char separator){  
//////////////////////////////////////////////////////////////////////////////

  vector<string> elements;
  vector<string> elements_with_tabs;
  stringstream ss(stringtodivide);
  string item;
  
  while(getline(ss, item, separator)) 
    if(item.size()) elements.push_back(item);

  // special case of space (a tab = a space)
  if(separator==' '){
    for(int i=0; i<(int)elements.size(); i++){
      stringstream ss_tab(elements[i]);
      while(getline(ss_tab, item, '\t')) 
	if(item.size()) elements_with_tabs.push_back(item);
    }
  }
  else return elements;

  return elements_with_tabs;
}

//////////////////////////////////////////////////////////////////////////////
string StringToUpper(string stringtoconvert){
//////////////////////////////////////////////////////////////////////////////
  string uppers="";
  for (string::iterator p = stringtoconvert.begin(); stringtoconvert.end() != p; ++p) uppers += toupper(*p);
  
  return uppers;
}

//////////////////////////////////////////////////////////////////////////////
string ReplaceAll(string str, const string& from, const string& to){
//////////////////////////////////////////////////////////////////////////////
  size_t start_pos = 0;
  while((start_pos = str.find(from, start_pos)) != std::string::npos) {
    str.replace(start_pos, from.length(), to);
    start_pos += to.length(); // Handles case where 'to' is a substring of 'from'
  }
  return str;
}

//////////////////////////////////////////////////////////////////////////////
bool IsDirectory(const char* dirname){
//////////////////////////////////////////////////////////////////////////////

  DIR* dp=NULL; dp = opendir(dirname);
  if(dp == NULL) return false;
  closedir(dp);

  return true;
}

//////////////////////////////////////////////////////////////////////////////
bool ListDirectories(const string maindir, vector <string> &subdir){
//////////////////////////////////////////////////////////////////////////////
  subdir.clear();
  DIR *dir;
  struct dirent *entry;

  if((dir = opendir(maindir.c_str())) == NULL){
    cerr<<"ListDirectories(): failed opening directory "<<maindir<<endl;
    return false;
  }
  else{
    if(closedir(dir) < 0){
      cerr<<"ListDirectories(): failed closing directory "<<maindir<<endl;
      return false;
    }
    else{
      dir = opendir(maindir.c_str());
      entry = readdir(dir);
      while (entry != NULL){
	if(IsDirectory(maindir+"/"+(string)entry->d_name)){
	  if(((string)(entry->d_name)).compare(".")&&((string)(entry->d_name)).compare(".."))
	    subdir.push_back((string)(entry->d_name));
	}
	entry = readdir(dir);
      }
      closedir(dir);
    }
  }
  
  return true;
}

//////////////////////////////////////////////////////////////////////////////
bool IsTextFile(const char* filename){
//////////////////////////////////////////////////////////////////////////////

  FILE* fp=NULL; fp = fopen(filename,"r");
  if(fp == NULL) return false;
  fclose(fp);

  return true;
}

//////////////////////////////////////////////////////////////////////////////
bool IsBinaryFile(const char* filename){
//////////////////////////////////////////////////////////////////////////////
  FILE* fp=NULL; fp = fopen(filename,"rb");
  if(fp == NULL) return false;
  fclose(fp);
  return true;
}

//////////////////////////////////////////////////////////////////////////////
vector<string>  Glob(const char* pattern){
//////////////////////////////////////////////////////////////////////////////

  glob_t globbuf;

  globbuf.gl_offs = 2;
  glob(pattern, GLOB_TILDE|GLOB_BRACE|GLOB_MARK, NULL, &globbuf);

  vector<string> results;
  for(unsigned i=0; i<globbuf.gl_pathc; i++) {
    results.push_back(globbuf.gl_pathv[i]);
  }
  
  globfree(&globbuf);
  return results;
}
